/*
 * Copyright (c) 2014 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Fri May 23 2014
 */
#include "ac_tk3_flash.h"

#include <getopt.h>
#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "flash.h"

static void	usage(FILE *channel, char *argv0);

static const char shortopts_string[] = "+ivh";
static struct option longopts_list[] = {
  { "init",		no_argument,		NULL,	'i'},
  { "version",		no_argument,		NULL,	'v'},
  { "help",		no_argument,		NULL,	'h'},
  { NULL, 0, NULL, 0}
};


/* --- main ---------------------------------------------------------------- */

int
main(int argc, char **argv)
{
  int init_p = 0;

  char *argv0 = argv[0];
  int c;

  /* parse command line options */
  while (
    (c = getopt_long(argc, argv, shortopts_string, longopts_list, NULL)) != -1)
    switch (c) {
      case 0: break;

      case 'i': init_p++; break;

      case 'v': puts(PACKAGE_VERSION); exit(0); break;
      case 'h':  usage(stdout, argv0);  exit(0); break;

      case '?':
      default:
	usage(stderr, argv0);
        exit(1);
        break;
    }
  argc -= optind;
  argv += optind;
  if (!argc) { usage(stderr, argv0); exit(1); }

  if (!strcmp(argv[0], "flash")) {
    if (argc < 3) { usage(stderr, argv0); exit(1); }

    if (mkbl_flash(argv[2], argv[1], 0, init_p)) exit(2);
    return 0;
  }

  if (!strcmp(argv[0], "set")) {
    if (argc < 2 || argc % 2) { usage(stderr, argv0); exit(1); }

    if (mkbl_params(argv[1], 0, &argv[2])) exit(2);
    return 0;
  }

  usage(stderr, argv0);
  return 1;
}


/* --- usage --------------------------------------------------------------- */

static void
usage(FILE *channel, char *argv0)
{
  fprintf(
    channel,
    "Usage:\n"
    "	%1$s [-i] flash <dev> <elf-file>\n"
    "		-i|--init		Reinitialize EEPROM with defaults\n"
    "\n"
    "	%1$s set <dev> [key value] [key value]...\n"
    "\n"
    "	%1$s -h|--help	Display this usage\n"
    "	%1$s -v|--version	Display version\n",
    basename(argv0));
}
